/* Copyright (c) 2016 SpaceToad and the BuildCraft team
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/. */
package buildcraft.silicon.gui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiButtonImage;
import net.minecraft.client.gui.recipebook.GuiRecipeBook;
import net.minecraft.client.gui.recipebook.IRecipeShownListener;
import net.minecraft.client.resources.I18n;
import net.minecraft.inventory.ClickType;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.util.ResourceLocation;

import net.minecraftforge.common.crafting.IShapedRecipe;

import buildcraft.api.core.BCLog;

import buildcraft.lib.gui.GuiBC8;
import buildcraft.lib.gui.GuiIcon;
import buildcraft.lib.gui.ledger.LedgerHelp;
import buildcraft.lib.gui.pos.GuiRectangle;
import buildcraft.lib.gui.recipe.GuiRecipeBookPhantom;
import buildcraft.lib.registry.TagManager;

import buildcraft.silicon.container.ContainerAdvancedCraftingTable;

public class GuiAdvancedCraftingTable extends GuiBC8<ContainerAdvancedCraftingTable> implements IRecipeShownListener {
    private static final ResourceLocation TEXTURE_BASE = new ResourceLocation("buildcraftsilicon:textures/gui/advanced_crafting_table.png");
    private static final ResourceLocation VANILLA_CRAFTING_TABLE = new ResourceLocation("textures/gui/container/crafting_table.png");
    private static final int SIZE_X = 176, SIZE_Y = 241;
    private static final GuiIcon ICON_GUI = new GuiIcon(TEXTURE_BASE, 0, 0, SIZE_X, SIZE_Y);
    private static final GuiIcon ICON_PROGRESS = new GuiIcon(TEXTURE_BASE, SIZE_X, 0, 4, 70);
    private static final GuiRectangle RECT_PROGRESS = new GuiRectangle(164, 7, 4, 70);

    private final GuiRecipeBookPhantom recipeBook;
    /** If true then the recipe book will be drawn on top of this GUI, rather than beside it */
    private boolean widthTooNarrow;
    private GuiButtonImage recipeButton;

    public GuiAdvancedCraftingTable(ContainerAdvancedCraftingTable container) {
        super(container);
        field_146999_f = SIZE_X;
        field_147000_g = SIZE_Y;
        GuiRecipeBookPhantom book;
        try {
            book = new GuiRecipeBookPhantom(this::sendRecipe);
        } catch (ReflectiveOperationException e) {
            BCLog.logger.warn("[silicon.gui] An exception was thrown while creating the recipe book gui!", e);
            book = null;
        }
        recipeBook = book;
        mainGui.shownElements.add(new LedgerHelp(mainGui, true));
    }

    private void sendRecipe(IRecipe recipe) {
        List<ItemStack> stacks = new ArrayList<>(9);

        int maxX = recipe instanceof IShapedRecipe ? ((IShapedRecipe) recipe).getRecipeWidth() : 3;
        int maxY = recipe instanceof IShapedRecipe ? ((IShapedRecipe) recipe).getRecipeHeight() : 3;
        int offsetX = maxX == 1 ? 1 : 0;
        int offsetY = maxY == 1 ? 1 : 0;
        List<Ingredient> ingredients = recipe.func_192400_c();
        if (ingredients.isEmpty()) {
            return;
        }
        for (int y = 0; y < 3; y++) {
            for (int x = 0; x < 3; x++) {
                if (x < offsetX || y < offsetY) {
                    stacks.add(ItemStack.field_190927_a);
                    continue;
                }
                int i = x - offsetX + (y - offsetY) * maxX;
                if (i >= ingredients.size() || x - offsetX >= maxX) {
                    stacks.add(ItemStack.field_190927_a);
                } else {
                    Ingredient ing = ingredients.get(i);
                    ItemStack[] matching = ing.func_193365_a();
                    if (matching.length >= 1) {
                        stacks.add(matching[0]);
                    } else {
                        stacks.add(ItemStack.field_190927_a);
                    }
                }
            }
        }

        container.sendSetPhantomSlots(container.tile.invBlueprint, stacks);
    }

    @Override
    protected boolean shouldAddHelpLedger() {
        // Don't add it on the left side because it clashes with the recipe book
        return false;
    }

    @Override
    public void func_73866_w_() {
        super.func_73866_w_();
        widthTooNarrow = this.field_146294_l < SIZE_X + 176;
        if (recipeBook != null) {
            InventoryCrafting invCraft = container.tile.getWorkbenchCrafting();
            recipeBook.func_194303_a(field_146294_l, field_146295_m, field_146297_k, widthTooNarrow, invCraft);
            field_147003_i = recipeBook.func_193011_a(widthTooNarrow, field_146294_l, field_146999_f);
            recipeButton =
                new GuiButtonImage(10, field_147003_i + 5, field_146295_m / 2 - 90, 20, 18, 0, 168, 19, VANILLA_CRAFTING_TABLE);
            field_146292_n.add(this.recipeButton);
        }
    }

    @Override
    public void func_73876_c() {
        super.func_73876_c();
        if (recipeBook != null) {
            recipeBook.func_193957_d();
        }
    }

    @Override
    public void func_73863_a(int mouseX, int mouseY, float partialTicks) {
        if (recipeBook == null) {
            super.func_73863_a(mouseX, mouseY, partialTicks);
            return;
        }
        if (recipeBook.func_191878_b() && this.widthTooNarrow) {
            func_146276_q_();
            this.func_146976_a(partialTicks, mouseX, mouseY);
            recipeBook.func_191861_a(mouseX, mouseY, partialTicks);
            func_191948_b(mouseX, mouseY);
        } else {
            super.func_73863_a(mouseX, mouseY, partialTicks);
            recipeBook.func_191861_a(mouseX, mouseY, partialTicks);
            recipeBook.func_191864_a(this.field_147003_i, this.field_147009_r, true, partialTicks);
        }

        recipeBook.func_191876_c(this.field_147003_i, this.field_147009_r, mouseX, mouseY);
    }


    @Override
    protected void drawBackgroundLayer(float partialTicks) {
        ICON_GUI.drawAt(mainGui.rootElement);

        long target = container.tile.getTarget();
        if (target != 0) {
            double v = (double) container.tile.power / target;
            ICON_PROGRESS.drawCutInside(
                    new GuiRectangle(
                            RECT_PROGRESS.x,
                            (int) (RECT_PROGRESS.y + RECT_PROGRESS.height * Math.max(1 - v, 0)),
                            RECT_PROGRESS.width,
                            (int) Math.ceil(RECT_PROGRESS.height * Math.min(v, 1))
                    ).offset(mainGui.rootElement)
            );
        }
    }

    @Override
    protected void drawForegroundLayer() {
        String title = I18n.func_135052_a("tile." + TagManager.getTag("block.advanced_crafting_table", TagManager.EnumTagType.UNLOCALIZED_NAME) + ".name");
        field_146289_q.func_78276_b(title, field_147003_i + (field_146999_f - field_146289_q.func_78256_a(title)) / 2, field_147009_r + 5, 0x404040);
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException {
        if (button == recipeButton && recipeBook != null) {
            recipeBook.func_193014_a(widthTooNarrow, container.tile.getWorkbenchCrafting());
            recipeBook.func_191866_a();
            field_147003_i = recipeBook.func_193011_a(widthTooNarrow, field_146294_l, field_146999_f);
            recipeButton.func_191746_c(this.field_147003_i + 5, this.field_146295_m / 2 - 90);
        }
    }

    @Override
    protected void func_73864_a(int mouseX, int mouseY, int mouseButton) throws IOException {
        if (recipeBook == null) {
            super.func_73864_a(mouseX, mouseY, mouseButton);
            return;
        }
        if (!recipeBook.func_191862_a(mouseX, mouseY, mouseButton)) {
            if (!widthTooNarrow || !recipeBook.func_191878_b()) {
                super.func_73864_a(mouseX, mouseY, mouseButton);
            }
        }
    }

    @Override
    protected void func_73869_a(char typedChar, int keyCode) throws IOException {
        if (recipeBook == null) {
            super.func_73869_a(typedChar, keyCode);
            return;
        }
        if (!recipeBook.func_191859_a(typedChar, keyCode)) {
            super.func_73869_a(typedChar, keyCode);
        }
    }

    @Override
    protected void func_184098_a(Slot slot, int slotId, int mouseButton, ClickType type) {
        super.func_184098_a(slot, slotId, mouseButton, type);
        if (recipeBook != null) {
            recipeBook.func_191874_a(slot);
        }
    }

    @Override
    protected boolean func_146978_c(int rectX, int rectY, int rectWidth, int rectHeight, int pointX, int pointY) {
        if (recipeBook == null) {
            return super.func_146978_c(rectX, rectY, rectWidth, rectHeight, pointX, pointY);
        }
        return (!widthTooNarrow || !recipeBook.func_191878_b())
            && super.func_146978_c(rectX, rectY, rectWidth, rectHeight, pointX, pointY);
    }

    @Override
    protected boolean func_193983_c(int mouseX, int mouseY, int _guiLeft, int _guiTop) {
        if (recipeBook == null) {
            return super.func_193983_c(mouseX, mouseY, _guiLeft, _guiTop);
        }
        boolean flag =
            mouseX < _guiLeft || mouseY < _guiTop || mouseX >= _guiLeft + field_146999_f || mouseY >= _guiTop + this.field_147000_g;
        return recipeBook.func_193955_c(mouseX, mouseY, field_147003_i, field_147009_r, field_146999_f, field_147000_g) && flag;
    }

    @Override
    public void func_146281_b() {
        if (recipeBook != null) {
            recipeBook.func_191871_c();
        }
        super.func_146281_b();
    }

    // IRecipeShownListener

    @Override
    public void func_192043_J_() {
        if (recipeBook != null) {
            recipeBook.func_193948_e();
        }
    }

    @Override
    public GuiRecipeBook func_194310_f() {
        return recipeBook;
    }
}
