/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.transport.stripes;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.block.BlockDispenser;
import net.minecraft.block.state.IBlockState;
import net.minecraft.dispenser.IBehaviorDispenseItem;
import net.minecraft.dispenser.IBlockSource;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import buildcraft.api.transport.IStripesActivator;
import buildcraft.api.transport.IStripesHandlerItem;

public enum StripesHandlerDispenser implements IStripesHandlerItem {
    INSTANCE;

    public static final List<Item> ITEMS = new ArrayList<>();
    public static final List<Class<? extends Item>> ITEM_CLASSES = new ArrayList<>();

    public static class Source implements IBlockSource {
        private final World world;
        private final BlockPos pos;
        private final EnumFacing side;

        public Source(World world, BlockPos pos, EnumFacing side) {
            this.world = world;
            this.pos = pos;
            this.side = side;
        }

        @Override
        public double func_82615_a() {
            return pos.func_177958_n() + 0.5D;
        }

        @Override
        public double func_82617_b() {
            return pos.func_177956_o() + 0.5D;
        }

        @Override
        public double func_82616_c() {
            return pos.func_177952_p() + 0.5D;
        }

        @Override
        public BlockPos func_180699_d() {
            return pos;
        }

        @Override
        public IBlockState func_189992_e() {
            return Blocks.field_150367_z.func_176223_P().func_177226_a(BlockDispenser.field_176441_a, side);
        }

        @SuppressWarnings("unchecked")
        @Override
        public <T extends TileEntity> T func_150835_j() {
            return (T) world.func_175625_s(pos);
        }

        @Override
        public World func_82618_k() {
            return world;
        }
    }

    private static boolean shouldHandle(ItemStack stack) {
        if (ITEMS.contains(stack.func_77973_b())) {
            return true;
        }

        Class<?> c = stack.func_77973_b().getClass();
        while (c != Item.class) {
            if (ITEMS.contains(c)) {
                return true;
            }
            c = c.getSuperclass();
        }
        return false;
    }

    @Override
    public boolean handle(World world,
                          BlockPos pos,
                          EnumFacing direction,
                          ItemStack stack,
                          EntityPlayer player,
                          IStripesActivator activator) {
        if (!BlockDispenser.field_149943_a.func_148741_d(stack.func_77973_b())) {
            return false;
        }
        IBehaviorDispenseItem behaviour = BlockDispenser.field_149943_a.func_82594_a(stack.func_77973_b());
        // Temp: for testing
        // if (!shouldHandle(stack)) {
        // return false;
        // }

        IBlockSource source = new Source(world, pos, direction);
        ItemStack output = behaviour.func_82482_a(source, stack.func_77946_l());
        player.field_71071_by.func_70299_a(player.field_71071_by.field_70461_c, output);
        return true;
    }
}
